@@ -25,7 +25,7 @@ float32Array[0] = -1; // 0xBF800000
 const bigEndian = uInt8Float32Array[3] === 0;
 
 function checkBounds(buf, offset, byteLength) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   if (buf[offset] === undefined || buf[offset + byteLength] === undefined)
     boundsError(offset, buf.length - (byteLength + 1));
 }
@@ -37,13 +37,9 @@ function checkInt(value, min, max, buf, offset, byteLength) {
   checkBounds(buf, offset, byteLength);
 }
 
-function checkNumberType(value, type) {
-  validateNumber(value, type || 'offset');
-}
-
 function boundsError(value, length, type) {
   if (Math.floor(value) !== value) {
-    checkNumberType(value, type);
+    validateNumber(value, type);
     throw new ERR_OUT_OF_RANGE(type || 'offset', 'an integer', value);
   }
 
@@ -56,6 +52,82 @@ function boundsError(value, length, type) {
 }
 
 // Read integers.
+function readBigUInt64LE(offset = 0) {
+  validateNumber(offset, 'offset');
+  const first = this[offset];
+  const last = this[offset + 7];
+  if (first === undefined || last === undefined)
+    boundsError(offset, this.length - 8);
+
+  const lo = first +
+    this[++offset] * 2 ** 8 +
+    this[++offset] * 2 ** 16 +
+    this[++offset] * 2 ** 24;
+
+  const hi = this[++offset] +
+    this[++offset] * 2 ** 8 +
+    this[++offset] * 2 ** 16 +
+    last * 2 ** 24;
+
+  return BigInt(lo) + (BigInt(hi) << 32n);
+}
+
+function readBigUInt64BE(offset = 0) {
+  validateNumber(offset, 'offset');
+  const first = this[offset];
+  const last = this[offset + 7];
+  if (first === undefined || last === undefined)
+    boundsError(offset, this.length - 8);
+
+  const hi = first * 2 ** 24 +
+    this[++offset] * 2 ** 16 +
+    this[++offset] * 2 ** 8 +
+    this[++offset];
+
+  const lo = this[++offset] * 2 ** 24 +
+    this[++offset] * 2 ** 16 +
+    this[++offset] * 2 ** 8 +
+    last;
+
+  return (BigInt(hi) << 32n) + BigInt(lo);
+}
+
+function readBigInt64LE(offset = 0) {
+  validateNumber(offset, 'offset');
+  const first = this[offset];
+  const last = this[offset + 7];
+  if (first === undefined || last === undefined)
+    boundsError(offset, this.length - 8);
+
+  const val = this[offset + 4] +
+    this[offset + 5] * 2 ** 8 +
+    this[offset + 6] * 2 ** 16 +
+    (last << 24); // Overflow
+  return (BigInt(val) << 32n) +
+    BigInt(first +
+      this[++offset] * 2 ** 8 +
+      this[++offset] * 2 ** 16 +
+      this[++offset] * 2 ** 24);
+}
+
+function readBigInt64BE(offset = 0) {
+  validateNumber(offset, 'offset');
+  const first = this[offset];
+  const last = this[offset + 7];
+  if (first === undefined || last === undefined)
+    boundsError(offset, this.length - 8);
+
+  const val = (first << 24) + // Overflow
+    this[++offset] * 2 ** 16 +
+    this[++offset] * 2 ** 8 +
+    this[++offset];
+  return (BigInt(val) << 32n) +
+    BigInt(this[++offset] * 2 ** 24 +
+      this[++offset] * 2 ** 16 +
+      this[++offset] * 2 ** 8 +
+      last);
+}
+
 function readUIntLE(offset, byteLength) {
   if (byteLength === 6)
     return readUInt48LE(this, offset);
@@ -74,7 +146,7 @@ function readUIntLE(offset, byteLength) {
 }
 
 function readUInt48LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 5];
   if (first === undefined || last === undefined)
@@ -88,7 +160,7 @@ function readUInt48LE(buf, offset = 0) {
 }
 
 function readUInt40LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 4];
   if (first === undefined || last === undefined)
@@ -102,7 +174,7 @@ function readUInt40LE(buf, offset = 0) {
 }
 
 function readUInt32LE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -115,7 +187,7 @@ function readUInt32LE(offset = 0) {
 }
 
 function readUInt24LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 2];
   if (first === undefined || last === undefined)
@@ -125,7 +197,7 @@ function readUInt24LE(buf, offset = 0) {
 }
 
 function readUInt16LE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 1];
   if (first === undefined || last === undefined)
@@ -135,7 +207,7 @@ function readUInt16LE(offset = 0) {
 }
 
 function readUInt8(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const val = this[offset];
   if (val === undefined)
     boundsError(offset, this.length - 1);
@@ -161,7 +233,7 @@ function readUIntBE(offset, byteLength) {
 }
 
 function readUInt48BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 5];
   if (first === undefined || last === undefined)
@@ -175,7 +247,7 @@ function readUInt48BE(buf, offset = 0) {
 }
 
 function readUInt40BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 4];
   if (first === undefined || last === undefined)
@@ -189,7 +261,7 @@ function readUInt40BE(buf, offset = 0) {
 }
 
 function readUInt32BE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -202,7 +274,7 @@ function readUInt32BE(offset = 0) {
 }
 
 function readUInt24BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 2];
   if (first === undefined || last === undefined)
@@ -212,7 +284,7 @@ function readUInt24BE(buf, offset = 0) {
 }
 
 function readUInt16BE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 1];
   if (first === undefined || last === undefined)
@@ -239,7 +311,7 @@ function readIntLE(offset, byteLength) {
 }
 
 function readInt48LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 5];
   if (first === undefined || last === undefined)
@@ -254,7 +326,7 @@ function readInt48LE(buf, offset = 0) {
 }
 
 function readInt40LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 4];
   if (first === undefined || last === undefined)
@@ -268,7 +340,7 @@ function readInt40LE(buf, offset = 0) {
 }
 
 function readInt32LE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -281,7 +353,7 @@ function readInt32LE(offset = 0) {
 }
 
 function readInt24LE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 2];
   if (first === undefined || last === undefined)
@@ -292,7 +364,7 @@ function readInt24LE(buf, offset = 0) {
 }
 
 function readInt16LE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 1];
   if (first === undefined || last === undefined)
@@ -303,7 +375,7 @@ function readInt16LE(offset = 0) {
 }
 
 function readInt8(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const val = this[offset];
   if (val === undefined)
     boundsError(offset, this.length - 1);
@@ -329,7 +401,7 @@ function readIntBE(offset, byteLength) {
 }
 
 function readInt48BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 5];
   if (first === undefined || last === undefined)
@@ -344,7 +416,7 @@ function readInt48BE(buf, offset = 0) {
 }
 
 function readInt40BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 4];
   if (first === undefined || last === undefined)
@@ -358,7 +430,7 @@ function readInt40BE(buf, offset = 0) {
 }
 
 function readInt32BE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -371,7 +443,7 @@ function readInt32BE(offset = 0) {
 }
 
 function readInt24BE(buf, offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = buf[offset];
   const last = buf[offset + 2];
   if (first === undefined || last === undefined)
@@ -382,7 +454,7 @@ function readInt24BE(buf, offset = 0) {
 }
 
 function readInt16BE(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 1];
   if (first === undefined || last === undefined)
@@ -394,7 +466,7 @@ function readInt16BE(offset = 0) {
 
 // Read floats
 function readFloatBackwards(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -408,7 +480,7 @@ function readFloatBackwards(offset = 0) {
 }
 
 function readFloatForwards(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 3];
   if (first === undefined || last === undefined)
@@ -422,7 +494,7 @@ function readFloatForwards(offset = 0) {
 }
 
 function readDoubleBackwards(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 7];
   if (first === undefined || last === undefined)
@@ -440,7 +512,7 @@ function readDoubleBackwards(offset = 0) {
 }
 
 function readDoubleForwards(offset = 0) {
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   const first = this[offset];
   const last = this[offset + 7];
   if (first === undefined || last === undefined)
@@ -458,6 +530,68 @@ function readDoubleForwards(offset = 0) {
 }
 
 // Write integers.
+function writeBigU_Int64LE(buf, value, offset, min, max) {
+  checkInt(value, min, max, buf, offset, 7);
+
+  let lo = Number(value & 0xffffffffn);
+  buf[offset++] = lo;
+  lo = lo >> 8;
+  buf[offset++] = lo;
+  lo = lo >> 8;
+  buf[offset++] = lo;
+  lo = lo >> 8;
+  buf[offset++] = lo;
+  let hi = Number(value >> 32n & 0xffffffffn);
+  buf[offset++] = hi;
+  hi = hi >> 8;
+  buf[offset++] = hi;
+  hi = hi >> 8;
+  buf[offset++] = hi;
+  hi = hi >> 8;
+  buf[offset++] = hi;
+  return offset;
+}
+
+function writeBigUInt64LE(value, offset = 0) {
+  return writeBigU_Int64LE(this, value, offset, 0n, 0xffffffffffffffffn);
+}
+
+function writeBigU_Int64BE(buf, value, offset, min, max) {
+  checkInt(value, min, max, buf, offset, 7);
+
+  let lo = Number(value & 0xffffffffn);
+  buf[offset + 7] = lo;
+  lo = lo >> 8;
+  buf[offset + 6] = lo;
+  lo = lo >> 8;
+  buf[offset + 5] = lo;
+  lo = lo >> 8;
+  buf[offset + 4] = lo;
+  let hi = Number(value >> 32n & 0xffffffffn);
+  buf[offset + 3] = hi;
+  hi = hi >> 8;
+  buf[offset + 2] = hi;
+  hi = hi >> 8;
+  buf[offset + 1] = hi;
+  hi = hi >> 8;
+  buf[offset] = hi;
+  return offset + 8;
+}
+
+function writeBigUInt64BE(value, offset = 0) {
+  return writeBigU_Int64BE(this, value, offset, 0n, 0xffffffffffffffffn);
+}
+
+function writeBigInt64LE(value, offset = 0) {
+  return writeBigU_Int64LE(
+    this, value, offset, -0x8000000000000000n, 0x7fffffffffffffffn);
+}
+
+function writeBigInt64BE(value, offset = 0) {
+  return writeBigU_Int64BE(
+    this, value, offset, -0x8000000000000000n, 0x7fffffffffffffffn);
+}
+
 function writeUIntLE(value, offset = 0, byteLength) {
   if (byteLength === 6)
     return writeU_Int48LE(this, value, offset, 0, 0xffffffffffff);
@@ -554,7 +688,7 @@ function writeUInt16LE(value, offset = 0) {
 function writeU_Int8(buf, value, offset, min, max) {
   value = +value;
   // `checkInt()` can not be used here because it checks two entries.
-  checkNumberType(offset);
+  validateNumber(offset, 'offset');
   if (value > max || value < min) {
     throw new ERR_OUT_OF_RANGE('value', `>= ${min} and <= ${max}`, value);
   }
@@ -777,6 +911,14 @@ module.exports = {
   setupBufferJS,
   // Container to export all read write functions.
   readWrites: {
+    readBigUInt64LE,
+    readBigUInt64BE,
+    readBigInt64LE,
+    readBigInt64BE,
+    writeBigUInt64LE,
+    writeBigUInt64BE,
+    writeBigInt64LE,
+    writeBigInt64BE,
     readUIntLE,
     readUInt32LE,
     readUInt16LE,
