@@ -47,9 +47,9 @@ function run(test) {
   REPL.createInternalRepl(env, opts, function(err, repl) {
     assert.ifError(err);
 
-    assert.strictEqual(expected.terminal, repl.terminal,
+    assert.strictEqual(repl.terminal, expected.terminal,
                        `Expected ${inspect(expected)} with ${inspect(env)}`);
-    assert.strictEqual(expected.useColors, repl.useColors,
+    assert.strictEqual(repl.useColors, expected.useColors,
                        `Expected ${inspect(expected)} with ${inspect(env)}`);
     repl.close();
   });
